package nl.cwi.sen1.tunit.example;
import nl.cwi.sen1.tunit.TUnitTestCase;
import nl.cwi.sen1.tunit.ToolComValidator;
import aterm.ATerm;
import aterm.pure.PureFactory;

/**
 * An example test case to demonstrate the toolbus testing framework.
 *
 * @author Bas Basten & Anton Lycklama a Nijeholt
 */
public class ExampleTestCase extends TUnitTestCase{
	
	public ExampleTestCase(){
		super();
	}
	
    /**
     * The function that contains the actual test case.
     */
    public void testExample(){
    	defaultTestToolExecutorFactory.addComValidatorForExecutableTool("executedTool", ExecutedTestTool.class);
    	
    	Tool1 tool1 = new Tool1(this, "tool1", true);
    	Tool2 tool2 = new Tool2(this, "tool2", true);
        
    	Thread tool1Executor = new Thread(tool1);
    	tool1Executor.setDaemon(true);
    	tool1Executor.start();
    	Thread tool2Executor = new Thread(tool2);
    	tool2Executor.setDaemon(true);
    	tool2Executor.start();
    	
    	toolbus.waitTillShutdown();
    	
    	if(hasFailed()) fail();
        
        System.out.println("Test Passed");
    }
    
    protected void setUp(){
    	try{
    		startToolbus("./src/nl/cwi/sen1/tunit/example/", "./src/nl/cwi/sen1/tunit/example/init.tb");
    	}catch(Exception ex){
    		ex.printStackTrace();
    		stopToolbus();
    		fail(ex.toString());
    	}
    }
    
    protected void tearDown(){
    	stopToolbus();
    }
    
    public static class Tool1 extends ToolComValidator{
    	private final TUnitTestCase testCase;
    	
    	public Tool1(TUnitTestCase testCase, String name, boolean verbose){
    		super(testCase, name, verbose);
    		
    		this.testCase = testCase;
    	}
    	
    	public void run(){
    		try{
    			testCase.connectToolComValidator(this);
    			
    			PureFactory factory = getFactory();
    			ATerm sendValue = factory.make("sendValue(<int>)", new Integer(10));
    			ATerm showValue = factory.make("showValue(<int>)", new Integer(10));
    			
    			sendEvent(sendValue);
    			
    			registerForDo(showValue);
    			expectAction();
    			
    			waitForCompletion();
    		}catch(Exception ex){
    			ex.printStackTrace();
    		}finally{
    			disconnect();
    		}
    	}
    }
    
    public static class Tool2 extends ToolComValidator{
    	private final TUnitTestCase testCase;
    	
    	public Tool2(TUnitTestCase testCase, String name, boolean verbose){
    		super(testCase, name, verbose);
    		
    		this.testCase = testCase;
    	}
    	
    	public void run(){
    		try{
    			testCase.connectToolComValidator(this);
    			
    			PureFactory factory = getFactory();
    			ATerm sendValue = factory.make("sendValue(<int>)", new Integer(10));
    			ATerm showValue = factory.make("showValue(<int>)", new Integer(10));
    			
    			registerForEval(showValue, showValue);
    			expectAction();
    			
    			sendEvent(sendValue);
    			
    			waitForCompletion();
    		}catch(Exception ex){
    			ex.printStackTrace();
    		}finally{
    			disconnect();
    		}
    	}
    }
    
    public static class ExecutedTestTool extends ToolComValidator{
    	
    	public ExecutedTestTool(TUnitTestCase testCase, String name, int id, boolean verbose){
    		super(testCase, name, id, verbose);
    	}
    	
    	public void run(){
    		try{
	    		registerForDo(getFactory().make("\"executed\""));
	            expectAction();
	            
	            waitForCompletion();
    		}catch(Exception ex){
    			ex.printStackTrace();
    		}finally{
    			disconnect();
    		}
    	}
    }
}
